{*****************************************************}
{                                                     }
{   TFileHV 1.2                                       }
{   Autor: Hamden Vogel                               }
{   E-mail: hamdenvogel@gmail.com                     }
{   http://www.hvogel.com.br                          }
{                                                     }
{*****************************************************}
{ Componente para Procura de Arquivos, strings dentro }
{  de Arquivos, e versionamento de Arquivos via FTP.  }
{                                                     }
{
{ Freeware. ltima atualizao: 22-09-2011            }
{                                                     }
{*****************************************************}





unit FileHV;




interface




uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, FileCtrl, ComCtrls,
  CommCtrl, StdCtrls, ShlObj, DB, DbClient, MidasLib, ShellApi, ZLIBArchive, IdFTP, StrUtils, Clipbrd;


const
  cVersion = '1.2';
  cDefaultSelectSomeItem = 'Favor Selecionar algum Item';
  cDefaultPromptIfDeleteCheckedItens = 'Confirma a Excluso dos Itens Selecionados ?';
  cTitleErrorMessage = 'Erro';
  cTitleQuestionMessage = 'Confirmao';
  cTitleInformationMessage = 'Informao';


type
  TFileAttrKind = (ffaReadOnly, ffaHidden, ffaSysFile, ffaVolumeID, ffaDirectory, ffaArchive, ffaAnyFile);
  TFileAttr = set of TFileAttrKind;


  TFileDateTimeKind = (fftCreationTime, fftLastAccessTime);
  TFileDateTime = set of TFileDateTimeKind;


  TManageFiles = (mfCopy, mfMove, mfDelete, mfRename, mfNothingToDo);


  TFileHV = class(TComponent)


private
  s: TStringList;
  sCheckedItens: TStringList;
  fListView: TListView;
  tempFTP: TIdFTP;
  fHost: string;
  fUserName: string;
  fPassWord: string;
  fPort: integer;
  fDefaultFolder: string;
  fInitialFTPFolder: string;
  fSubFolder: boolean;
  fMoreFileDetails: Boolean;
  fShowLinePosition: Boolean;
  fAttr: TFileAttr;
  fDateTime: TFileDateTime;
  fPath: string;
  fFileMask: string;
  fDelimiter: string;
  fSearchString: string;
  fStrLinePosition: string;
  fCheckedItem: string;
  bPopulating: Boolean;
  iFoundLine: integer;
  Memo: TMemo;
  Line: integer;
  sFile: string;
  tempClientDataSet: TClientDataSet;
  K: integer;
  fVersion: string;
  fDefaultSelectSomeItem: string;
  fDefaultPromptIfDeleteCheckedItens: string;
  fTitleErrorMessage: string;
  fTitleQuestionMessage: string;
  fTitleInformationMessage: string;


  function UppercaseWordAndRemoveAccents(const Str: string): string;
  procedure CheckIfExistsAndDelete(const strFile: string);
  procedure SetListView(const Value: TListView);
  procedure CreateListView;
  procedure FillListView(Path: String);
  procedure SetCheckedItem(const Value: string);
  procedure fListViewDblClick(Sender: TObject);
  procedure FormShow(Sender: TObject);
  procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  procedure SetPath(Value: string);
  procedure FileSearch(const inPath : string);
  {funes internas utilitrias para utilizar o TClientDataSet temporrio}
  procedure CreateTempClientDataSet;
  function TempClientDataSetExists: Boolean;
  procedure RemoveFields;
  procedure DestroyTempClientDataSet;
  {fim funes TClientDataSet}
  procedure CreateResourceFile;
  procedure CompileDelphi;
  procedure DeleteTempFiles;
  function IsResourceFileAlreadyCreated: Boolean;
  procedure ExtractFilesFromXML(const EnteredDirectory: string = '');
  function extractResFromPas(binresname: string; path: string): boolean;
  procedure executeAndWait(fFile: String);
  procedure ManageFiles(const strReceiver: string; manageFileType: TManageFiles);
  function NothingChecked: Boolean;
  procedure SetVersion(const Value: String);
  procedure SetDefaultSelectSomeItem(const Value: string);
  procedure SetDefaultPromptIfDeleteCheckedItens(const Value: string);
  procedure SetTitleErrorMessage(const Value: string);
  procedure SetTitleQuestionMessage(const Value: string);
  procedure SetTitleInformationMessage(const Value: string);
  procedure SetHost(const Value: string);
  procedure SetUsername(const Value: string);
  procedure SetPassWord(const Value: string);
  procedure SetPort(const Value: integer);
  procedure SetDefaultFolder(const Value: string);
  procedure SetInitialFTPFolder(const Value: string);
  procedure DeleteProjectFolderNameFromFTPList;
  procedure FillFTPListView(const i: integer; const bDontShowFoldersName: Boolean = True);
  procedure AddColumnTipo;
  procedure ShowHistoryFiles(const myFTPFile: string = ''; const bDontShowDefaultFolder: Boolean = True);


  function NextFilename(aFilename : string) : string;
  function GetTmpDir: string; // Obter o diretrio temporrio do Windows
  function ComparaArquivos(Arq1,Arq2 : String): integer;
  function FileTimeToDTime(FTime: TFileTime): TDateTime;
  function FileModifiedDate(const strFullPathFile: string): TDateTime;
  function splitstr(valor, caracter: string; index: integer): string;
  function RemovePrefixosNumericosDeUmaString(const S: string): string;
  procedure ShowFiles;
  procedure DelFTPFile(const FTPFile: string);
  procedure RenFTPFile(const FTPFile: string; const FTPFileToBeRenamed: string);
  procedure RemoveDotsFromStringList;
  function LastFileVersion(const ffFile: string): string;




public
   ffullpathfile, ffile, lastversionffile, fdrive, ffolder, ffnextfile, tempZipFile, fProjectFolder: string;
   sReturnedItensFromFTPServer: TStringList;
  FileItens: TStringList;
  constructor Create(AOwner: TComponent); override;
  destructor Destroy; override;
  function SearchForFiles: TStringList;
  function SearchForString(const FileName: string; const StrFile: string): integer;
  function SelectDir: string;

  function IsSomeItemChecked: Boolean;
  procedure DeleteCheckedItens(const Physically: Boolean = False);
  procedure CopyFiles(const sReceiver: string);
  procedure MoveFiles(const sReceiver: string);
  procedure DeleteFiles(const sReceiver: string);
  procedure AddFileToBeCompacted(const Physically: Boolean = True);
  procedure ExtractCompactedFiles(const EnteredDirectory: string = '');
  function ListViewSelectedItem: string;
  function CheckedItens: TStringList;
  procedure CheckedAll;
  procedure UnCheckedAll;

  function LoadFTP: Boolean;
  function GetInfoFile(const myFullPathFile: string = ''): Boolean;
  function GetLastFileVersion: string;
  procedure Upload;
  procedure DownloadLastVersion;
  procedure GetFile(const MyFTPFile: string);
  procedure DeleteLastVersion;
  procedure DeleteFTPFile(const myFTPFile: string);
  function RenameFTPFile(const myFTPFile: string; const FTPFileToBeRenamed: string): Boolean;
  procedure LoopAllFTPFiles(const myFTPFile: string; const manageFileType: TManageFiles; const FTPFileToBeRenamed: string = '');
  procedure ShowFoldersAndFiles;
  procedure ShowAllFTPFiles;
  procedure ShowWorkingFTPFolders;
  procedure ShowWorkingFTPFoldersAndFiles;
  procedure ShowHistoricFiles(const myFTPFile: string = ''; const Preview: Boolean = False);
  procedure ShowAllVersionedFiles(const myFTPFile: string; const Preview: Boolean = False; const OriginalMyFTPFile: string = '');
  procedure SetInitialFolders;
  function RemovePrefixAndExtension(const MyFTPFile: string): string;


published
  property FileAttr:            TFileAttr       read fAttr                write fAttr;
  property FileDateTime:        TFileDateTime   read fDateTime            write fDateTime;
  property InSubFolders:        boolean         read fSubFolder           write fSubFolder;
  property MoreDetailsFile:     Boolean         read fMoreFileDetails     write fMoreFileDetails    default False;
  property Path:                string          read fPath                write SetPath;
  property FileMask:            string          read fFileMask            write fFileMask;
  property Delimiter:           string          read fDelimiter           write fDelimiter;
  property ListView:            TListView       read fListView            write SetListView;
  property SearchString:        string          read fSearchString        write fSearchString;
  property ShowLinePosition:    Boolean         read fShowLinePosition    write fShowLinePosition   default False;
  property StrLinePosition:     String          read fStrLinePosition     write fStrLinePosition;
  property Version:             string          read fVersion             write SetVersion;
  property DefaultSelectSomeItem: string        read fDefaultSelectSomeItem write SetDefaultSelectSomeItem;
  property DefaultPromptIfDeleteCheckedItens: string read fDefaultPromptIfDeleteCheckedItens write SetDefaultPromptIfDeleteCheckedItens;
  property TitleErrorMessage:     string        read fTitleErrorMessage   write SetTitleErrorMessage;
  property TitleQuestionMessage:  string        read fTitleQuestionMessage write SetTitleQuestionMessage;
  property TitleInformationMessage: string      read fTitleInformationMessage write SetTitleInformationMessage;
  property FTPPort:               integer       read fPort                  write SetPort          default 21;
  property FTPHost:               string        read fHost                  write SetHost;
  property FTPUserName:           string        read fUserName              write SetUsername;
  property FTPPassWord:           string        read fPassWord              write SetPassWord;
  property FTPDefaultFolder:      string        read fDefaultFolder         write SetDefaultFolder;
  property FTPInitialFolder:      string        read fInitialFTPFolder      write SetInitialFTPFolder;

end;




procedure Register;
{$R DCC32_.res}


implementation


const XMLFileName = 'Tempfile.xml'; //arquivo utilizado para o processo de gerao do arquivo de
//recurso, para o processo de anexao e extrao de arquivos.
const RESTempFile = 'temp.res'; //nome do arquivo de recursos.


procedure Register;
begin
  RegisterComponents('Additional', [TFileHV]);
end;




function FileTimetoDateTime(FtTime : TFileTime) : TDateTime;
var
  LocalTime : TFileTime;
  SystemTime : TSystemTime;
begin
  try
// Converte hora para hora local
    FileTimeToLocalFileTime(FtTime, LocalTime);
// Converte para TSystemTime
    FileTimeToSystemTime(LocalTime, SystemTime);
// Converte para TDateTime
    with SystemTime do
      Result := EncodeDate(wYear,wMonth,wDay) + EncodeTime(wHour,wMinute,wSecond,wMilliseconds);
  except
    Result := Date;
  end;
end;



constructor TFileHV.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Path                                := IncludeTrailingPathDelimiter(GetCurrentDir);
  fDefaultSelectSomeItem              := cDefaultSelectSomeItem;
  fDefaultPromptIfDeleteCheckedItens  := cDefaultPromptIfDeleteCheckedItens;
  fTitleErrorMessage                  := cTitleErrorMessage;
  fTitleQuestionMessage               := cTitleQuestionMessage;
  fTitleInformationMessage            := cTitleInformationMessage;
  FileMask                            := '*.*';
  FileAttr                            := [ffaAnyFile];
  fDelimiter                          := ' - ';
  fSearchString                       := '';
  fStrLinePosition                    := ' Line: '; //linha da ocorrncia encontrada.
  s                                   := TStringList.Create;
  sCheckedItens                       := TStringList.Create;
  sReturnedItensFromFTPServer         := TStringList.Create;
  iFoundLine                          := 0;
  Line                                := 0;
  sFile                               := '';
  k                                   := 0;
  fDefaultFolder                      := 'Sources'; //nome do Diretrio-Pai (servidor FTP).
  fProjectFolder                      := fDefaultFolder;
  fInitialFTPFolder                   := '';
  FileItens                           := TStringList.Create;
end;




destructor TFileHV.Destroy;
begin
  FreeAndNil(s);
  FreeAndNil(sCheckedItens);
  FreeAndNil(sReturnedItensFromFTPServer);
  FreeAndNil(FileItens);
  if Assigned(fListView) then fListView := nil;
  if Assigned(tempClientDataSet) then FreeAndNil(tempClientDataSet);
  if Assigned(tempFTP) then
  begin
    tempFTP.Disconnect;
    FreeAndNil(tempFTP);
  end;
  inherited Destroy;
end;




procedure TFileHV.SetPath (Value: string);
begin
  if fPath <> Value then
  begin
    if Value <> '' then
      if DirectoryExists(Value) then
        fPath := IncludeTrailingPathDelimiter(Value);
  end;
end;



//procurar por arquivos. Se tiver um ListView associado  ele,
//ele tambm ser preenchido.
function TFileHV.SearchForFiles: TStringList;
begin
  s.Clear;
  try
    FileSearch(Path);
    if Assigned(fListView) then //utiliza o ListView e preenche os dados.
    begin
      CreateListView;
      FillListView(Path +  fFileMask);
      bPopulating := False;
    end;
  finally
    Result := s;
  end;
end;


//procedure para localizar arquivos, tambm pode pesquisar por subdiretrios.
procedure TFileHV.FileSearch(const InPath : string);
function MoreDateTimeInfo(const Rec: TSearchRec): string;
begin
  if (fftCreationTime in FileDateTime) and not (fftLastAccessTime in FileDateTime) then
    Result := fDelimiter + DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftCreationTime))
  else if (fftCreationTime in FileDateTime) and (fftLastAccessTime in FileDateTime) then
    Result := fDelimiter + DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftCreationTime)) + fDelimiter +
    DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftLastAccessTime))
  else if not (fftCreationTime in FileDateTime) and (fftLastAccessTime in FileDateTime) then
    Result := fDelimiter + DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftLastAccessTime))
end;


var Rec: TSearchRec;
  Attr, LineSearch: integer;
  strPathAndFile, strFile: string;
begin
  Attr := 0;
  if ffaReadOnly in FileAttr then Attr := Attr + SysUtils.faReadOnly;
  if ffaHidden in FileAttr then Attr := Attr + faHidden;
  if ffaSysFile in FileAttr then Attr := Attr + faSysFile;
  if ffaVolumeID in FileAttr then Attr := Attr + faVolumeID;
  if ffaDirectory in FileAttr then Attr := Attr + faDirectory;
  if ffaArchive in FileAttr then Attr := Attr + faArchive;
  if ffaAnyFile in FileAttr then Attr := Attr + faAnyFile;


  if SysUtils.FindFirst(inPath + FileMask, Attr, Rec) = 0 then
  try
  repeat
    strPathAndFile := inPath + Rec.Name;
    if fSearchString = '' then
    begin
      if fMoreFileDetails then
        s.Add(strPathAndFile + MoreDateTimeInfo(Rec))
      else s.Add(strPathAndFile);
    end
    else
    begin
      LineSearch := SearchForString(strPathAndFile, fSearchString);
      if LineSearch > 0 then
      if fMoreFileDetails then
        strFile := strPathAndFile + MoreDateTimeInfo(Rec)
      else strFile := strPathAndFile;

      if (fShowLinePosition) and (LineSearch > 0) then
        s.Add(strFile + fStrLinePosition + inttostr(LineSearch))
      else if LineSearch > 0 then
        s.Add(strFile);
    end


  until SysUtils.FindNext(Rec) <> 0;
  finally
    SysUtils.FindClose(Rec);
  end;

  if not InSubFolders then Exit;

  if SysUtils.FindFirst (inPath + '*.*', faDirectory, Rec) = 0 then
  try
    repeat
      if ((Rec.Attr and faDirectory) <> 0) and (Rec.Name<>'.') and (Rec.Name<>'..') then
        FileSearch(IncludeTrailingPathDelimiter(inPath + Rec.Name));

    until SysUtils.FindNext(Rec) <> 0;
  finally
      SysUtils.FindClose(Rec);
  end;
end;


function TFileHV.SearchForString(const FileName: string; const StrFile: string): integer;
var
  FileToFind: TextFile;
  strFound: string;
  iCounter: integer;
begin
  Result   := 0;
  iCounter := 0;
  if not SysUtils.FileExists(FileName) then Exit;


  AssignFile(FileToFind,FileName);
  {i-}
  Reset(FileToFind);
  {i+}
  if (IOResult <> 0) then Abort;
    while not Eof(FileToFind) do
    begin
      Readln(FileToFind, strFound);
      Inc(iCounter);
      if (SysUtils.AnsiPos(UppercaseWordAndRemoveAccents(StrFile), UppercaseWordAndRemoveAccents(strFound)) <> 0) then //encontrou string no arquivo
      begin
        Result := iCounter;
        Break;
      end;
      Application.ProcessMessages;
    end;
  System.Close(FileToFind);
end;


//funo para remover acentos e transformar em maisculo.
function TFileHV.UppercaseWordAndRemoveAccents(
  const Str: string): string;
function RemoveAccents(S: string): string;
var
  I, L: integer;
  Buffer: array[0..1023] of WideChar;
begin
  L := MultiByteToWideChar (CP_ACP, MB_COMPOSITE, PChar (S), Length (S), Buffer,
        SizeOf(Buffer) div SizeOf (WideChar) {2});
  Result := '';
  for I := 0 to L - 1 do if Buffer [I] <= #255 then
    Result := Result + Buffer [I];
end;
begin
  Result := SysUtils.AnsiUpperCase(RemoveAccents(Str));
end;


procedure TFileHV.SetListView(const Value: TListView);
begin
  fListView := Value;
end;


procedure TFileHV.CreateListView;
var
  NewColumn: TListColumn;
begin
  if not Assigned(fListView) then exit;
  fListView.OnDblClick := fListViewDblClick;
  //fListView.Items.BeginUpdate;
  fListView.Items.Clear;
  fListView.Columns.Clear;
  bPopulating := True;


  with fListView do
  begin
    MultiSelect := False;
    Color :=   $00D8FEFC;
    ViewStyle := vsReport;
    ShowColumnHeaders := True;
    ShowHint := False;
    RowSelect := False;
    GridLines := False;
    CheckBoxes := True;
    NewColumn := Columns.Add;
    NewColumn.Caption := 'NomeArq';
    NewColumn.Width := 200;
    NewColumn := Columns.Add;
    NewColumn.Caption := 'Diretrio';
    NewColumn.Width := 250;
    NewColumn := Columns.Add;
    NewColumn.Caption := 'Tamanho';
    NewColumn.Width := 70;
    NewColumn := Columns.Add;
    NewColumn.Caption := 'Dt ltimo Acesso';
    NewColumn.Width := 150;


    if (fSearchString <> '') then
    begin
      NewColumn := Columns.Add;
      NewColumn.Caption := 'Encontrado:';
      NewColumn.Width := 70;
    end;

  end;
end;


procedure TFileHV.FillListView(Path: String);
function MoreDateTimeInfo(const Rec: TSearchRec): string;
begin
  if (fftCreationTime in FileDateTime) and not (fftLastAccessTime in FileDateTime) then
    Result := fDelimiter + DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftCreationTime))
  else if (fftCreationTime in FileDateTime) and (fftLastAccessTime in FileDateTime) then
    Result := fDelimiter + DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftCreationTime)) + fDelimiter +
    DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftLastAccessTime))
  else if not (fftCreationTime in FileDateTime) and (fftLastAccessTime in FileDateTime) then
    Result := fDelimiter + DateTimeToStr(FileTimetoDateTime(Rec.FindData.ftLastAccessTime))
end;


var
  Diretorio,Nome, strFile: String;
  SearchRec : TSearchRec;
  Done : Integer;
  LineSearch: Integer;
begin
// guarda nome do diretrio
  Diretorio := ExtractFilePath(Path);
// guarda nome do arquivo
  Nome := ExtractFileName(Path);
  Done := FindFirst(Path,0,SearchRec);


  while Done = 0 do
  begin
// encontrou
    with fListView.Items.Add,SearchRec.FindData do
    begin
      Caption := String(cFileName);
      SubItems.Add(Diretorio);
      SubItems.Add(IntToStr(nFileSizeLow));
      SubItems.Add(DateTimeToStr(FileTimetoDateTime(ftLastAccessTime)));


      if (fSearchString <> '') then
      begin
        LineSearch := SearchForString(Diretorio + Caption, fSearchString);
        if LineSearch > 0 then
        if fMoreFileDetails then
          strFile := Caption + MoreDateTimeInfo(SearchRec)// s.Add(strPathAndFile + MoreDateTimeInfo(Rec))
        else strFile := Caption;//  s.Add(strPathAndFile);

        if (fShowLinePosition) and (LineSearch > 0) then
          SubItems.Add(fStrLinePosition + inttostr(LineSearch))
      end;

   //   SubItems.Add(DateTimeToStr(FileTimetoDateTime(ftLastAccessTime)));
   //   SubItems.Add(DateTimeToStr(FileTimetoDateTime(ftLastWriteTime)));
   //   SubItems.Add(String(cAlternateFileName));
    end;

// procura prximo arquivo
    Done := FindNext(SearchRec);
  end;
  FindClose(SearchRec);

  if not InSubFolders then Exit;

  if SysUtils.FindFirst (Diretorio + '*.*', faDirectory, SearchRec) = 0 then
  try
    repeat
      if ((SearchRec.Attr and faDirectory) <> 0) and (SearchRec.Name<>'.') and (SearchRec.Name<>'..') then
        FillListView(IncludeTrailingPathDelimiter(Diretorio + SearchRec.Name) + FileMask);

    until SysUtils.FindNext(SearchRec) <> 0;
  finally
      SysUtils.FindClose(SearchRec);
  end;
end;


procedure TFileHV.SetCheckedItem(const Value: string);
begin
  fCheckedItem := Value;
end;


//duplo-clique no listview associado, para abrir uma tela
//contendo a linha encontrada com a string.
procedure TFileHV.fListViewDblClick(Sender: TObject);
function after(const Search, Find: string): string;
{Retorna uma cadeia de caracteres aps a parte da string selecionada}
var
  index: byte;
begin
  index := Pos(Find, Search);
  if index = 0 then
  begin
    Result := '';
  end
  else
  begin
    Result := Copy(Search, index + Length(Find), 255);
  end;
end;

var
  hts: THitTests;
  ListViewCursosPos: TPoint;
  selectedItem: TListItem;
  Frm: TForm;
begin
  //obter a posio do cursor do mouse relacionado ao  fListView
  ListViewCursosPos := fListView.ScreenToClient(Mouse.CursorPos);


  //verifica se houve o evento de duplo-clique (se foi disparado)
  hts := fListView.GetHitTestInfoAt(ListViewCursosPos.X, ListViewCursosPos.Y);


  //localiza o item no fListViem que foi clicado e retorna os dados dessa linha
  if hts <= [htOnIcon, htOnItem, htOnLabel, htOnStateIcon] then
  begin
    selectedItem := fListView.Selected;
    if Assigned(selectedItem) then
    begin
      if (fSearchString <> '') and (selectedItem.SubItems.Count = 4) then
      begin
        iFoundLine := StrToIntDef(trim(after(selectedItem.SubItems.Strings[3], ':')), 0);
        sFile := selectedItem.SubItems.Strings[0] + selectedItem.Caption;
        if not SysUtils.FileExists(sFile) then Exit;


        Frm := TForm.Create(nil);
        try
          Frm.Width := 500;
          Frm.Height := 400;
          Frm.KeyPreview := True;
         // Frm.BorderStyle := bsToolWindow;
          Frm.Position := poScreenCenter;
          Frm.Caption := 'Exibindo Linha Encontrada no Arquivo ' + SysUtils.ExtractFileName(sFile);
          Memo := TMemo.Create(nil);
          try
            Memo.Parent := Frm;
            Memo.Align  := alClient;
            //Memo.ReadOnly := True;
            Memo.ScrollBars := ssVertical;
            Line := (iFoundLine + 1);
            Frm.OnShow := FormShow;
            Frm.OnKeyDown := FormKeyDown;
            Frm.ShowModal;  

          finally
            Memo.Free;
          end;
        finally
          Frm.Free;
        end;
      end;
    end;
  end;
end;




procedure TFileHV.FormShow(Sender: TObject);
var
  i: integer;
begin
  Memo.Lines.Clear;
  Memo.Lines.LoadFromFile(sFile);
  Memo.SetFocus;
  with Memo do
  begin
    Line := 0;
    for i := 0 to Memo.Lines.Count - 1 do
    begin
      if System.Pos(UppercaseWordAndRemoveAccents(fSearchString), UppercaseWordAndRemoveAccents(Memo.Lines.Strings[i])) > 0 then
        break;
      inc(Line);
    end;

    //O cdigo abaixo selecionar a linha da string encontrada.
    //O ndice da linha do Memo ser posicionada pelo cursor do mouse e
    //selecionada como se o usurio selecionasse manualmente.
    SelStart := Perform(EM_LINEINDEX, Line, 0);
    Perform(EM_SCROLLCARET, 0, 0);
    keybd_event(ord(' '), 0, 0, 0);
    keybd_event(VK_BACK, 0, 0, 0);

     for i := 0 to length(Lines.Strings[Line]) do
     begin
       keybd_event(VK_SHIFT, 0, KEYEVENTF_EXTENDEDKEY or 0, 0);
       keybd_event(39, 0, 0, 0);
     end;                         

    keybd_event(VK_SHIFT, $45, KEYEVENTF_EXTENDEDKEY or KEYEVENTF_KEYUP, 0);
  end;
end;


procedure TFileHV.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = 27 then TForm(Sender).Close; //permite fechar a tela da linha encontrada com a tecla Esc
end;                              


function BrowseCallback(Wnd: HWND; uMsg: UINT; lParam, lpData: LPARAM):
   Integer; stdcall;
var
  Path : String;
  PathName : Array[0..MAX_PATH] of char;
  SRec : TSearchRec;
begin
  if uMsg = BFFM_INITIALIZED then
  begin
  // Inicializao - configura a pasta selecionada para a atual
    Path := ExtractFilePath(ParamStr(0));
    SendMessage(Wnd, BFFM_SETSELECTION, -1, LongInt(PChar(Path)));
  end
  else if uMsg = BFFM_SELCHANGED then
  begin
   // Mudana de pasta - pega o nome da pasta atual
    ShGetPathFromIdList(PItemIdList(lParam),@PathName);
    Path := IncludeTrailingPathDelimiter(String(PathName))+'*.*';
   // existem arquivos ?
    if FindFirst(Path,0,SRec) = 0 then
      SendMessage(Wnd, BFFM_ENABLEOK, 0, 1)
    else
      SendMessage(Wnd, BFFM_ENABLEOK, 0, 0);
// coloca nome da pasta na linha de status
    SendMessage(Wnd,BFFM_SETSTATUSTEXT, 0, LongInt(@PathName[0]));
  end;
  Result := 0;
end;


//Selecionar o diretrio para realizar as operaes de FTP.
function TFileHV.SelectDir: string;
var
  BrowseInfo : TBrowseInfo;
  PathName, DisplayName : Array[0..MAX_PATH] of Char;
  IdList : PItemIDList;
begin
  Result := '';
  with BrowseInfo do
  begin
    hwndOwner := Application.Handle;
    pidlRoot := nil;
    pszDisplayName := DisplayName;
    lpszTitle := 'Selecione o diretrio';
    ulFlags := BIF_STATUSTEXT;
    lpfn := BrowseCallback;
    lParam := 0;
  end;
  IDList := ShBrowseForFolder(BrowseInfo);
  if IDList <> nil then
  begin
    ShGetPathFromIdList(IDList,@PathName);
    Result := String(PathName);
    GlobalFreePtr(IDList);
  end;
end;


function TFileHV.NothingChecked: Boolean;
var
  i: integer;
begin
  Result := True;
  if not Assigned(fListView) then Exit;

  for i := 0 to Pred(fListView.Items.Count) do
  if fListView.Items[i].Checked then
  begin
    Result := False;
    Break;
  end;
end;


procedure TFileHV.CheckedAll;
var
  i: integer;
begin
  if not Assigned(fListView) then Exit;

  for i := 0 to Pred(fListView.Items.Count) do
    fListView.Items[i].Checked := True;
end;


procedure TFileHV.UnCheckedAll;
var
  i: integer;
begin
  if not Assigned(fListView) then Exit;

  for i := 0 to Pred(fListView.Items.Count) do
    fListView.Items[i].Checked := False;
end;





function TFileHV.IsSomeItemChecked: Boolean;
begin
  Result := not NothingChecked;
  if not Result then
    Application.MessageBox(PChar(fDefaultSelectSomeItem),'Erro',MB_OK + MB_ICONERROR);
end;


procedure TFileHV.DeleteCheckedItens(const Physically: Boolean = false);
var
  i : Integer;
begin
  if not IsSomeItemChecked then Exit;

  if Application.MessageBox(PChar(fDefaultPromptIfDeleteCheckedItens),
   PChar(fTitleQuestionMessage),mb_YesNo + mb_IconQuestion + mb_DefButton1) <> idYes then Exit;

   i := 0;
   while i <= fListView.Items.Count -1  do
   begin
     if fListView.Items[i].Checked then
     begin
       if (Physically) then SysUtils.DeleteFile(fListView.Items[i].SubItems[0]+fListView.Items[i].Caption);

       fListView.Items[i].Delete;
       i := i - 1;
     end;
     i := i + 1;
   end;
end;


procedure TFileHV.CreateTempClientDataSet;
begin
  if not Assigned(tempClientDataSet) then tempClientDataSet := TClientDataSet.Create(Self);

  if tempClientDataSet.Active then
    tempClientDataSet.EmptyDataSet;

  RemoveFields;

  with tempClientDataSet.FieldDefs do
  begin
    with AddFieldDef do
    begin
      Name := 'SQ_DOC';
      DataType := ftAutoInc;
    end;


    with AddFieldDef do
    begin
      Name := 'CONTENTS_DOC';
      DataType := ftBlob;
    end;


    with AddFieldDef do
    begin
      Name := 'TITLE_DOC';
      DataType := ftString;
      Size := 100;
    end;


    with AddFieldDef do
    begin
      Name := 'PATH_DOC';
      DataType := ftString;
      Size := 1000;
    end;
  end;


  if tempClientDataSet.Active then tempClientDataSet.Close;
  tempClientDataSet.CreateDataSet;
  tempClientDataSet.Edit;
  tempClientDataSet.Fields[0].AsString := '1';
  tempClientDataSet.Post;
  tempClientDataSet.Open;
  if SysUtils.FileExists(ExtractFilePath(Application.ExeName) + XMLFileName) then
    tempClientDataSet.LoadFromFile(ExtractFilePath(Application.ExeName)+ XMLFileName);
  //tempClientDataSet.EmptyDataSet;
end;


procedure TFileHV.executeAndWait(fFile: String);
var
  ShellExecuteInfo: TShellExecuteInfo;
begin
// limpa a estrutura
  FillChar(ShellExecuteInfo,Sizeof(TShellExecuteInfo),0);
  with ShellExecuteInfo do
  begin
    cbSize := SizeOf(ShellExecuteInfo);
// esta mscara  necessria para que o processo no seja terminado imediatamente
    fMask := SEE_MASK_NOCLOSEPROCESS;
    wnd := Application.MainForm.Handle;
    lpVerb := 'open';
    lpFile := PChar(fFile);
    lpParameters := nil;
    lpDirectory := nil;
    nShow := SW_HIDE;
  end;
// executa
  ShellExecuteEx(@ShellExecuteInfo);
// espera o final da execuo
  if ShellExecuteInfo.hProcess > 0 then
    WaitForSingleObject(ShellExecuteInfo.hProcess,INFINITE);
end;


function TFileHV.extractResFromPas(binresname, path: string): boolean;
var
  ResSize, HG, HI, SizeWritten, hFileWrite: Cardinal;
begin
  result := false;
  HI := FindResource(hInstance, @binresname[1], RT_RCDATA);
  if HI <> 0 then
  begin
    HG := LoadResource(hInstance, HI);
    if HG <> 0 then
    begin
      ResSize := SizeOfResource(hInstance, HI);
      hFileWrite := CreateFile(@path[1], GENERIC_READ or GENERIC_WRITE, FILE_SHARE_READ or FILE_SHARE_WRITE, nil,
      CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, 0);
    if hFileWrite <> INVALID_HANDLE_VALUE then
      try
        result := (WriteFile(hFileWrite, LockResource(HG)^, ResSize, SizeWritten, nil) and (SizeWritten = ResSize));
      finally
        CloseHandle(hFileWrite);
      end;
    end;
  end;
end;


procedure TFileHV.RemoveFields;
var
  i: integer;
begin
  if tempClientDataSet.FieldDefs.Count = 0 then Exit;
                 
  for i := 0 to tempClientDataSet.FieldCount - 1 do
    tempClientDataSet.FieldDefs.Find(tempClientDataSet.Fields[i].DisplayName).Free;

  tempClientDataSet.Close;
end;


function TFileHV.TempClientDataSetExists: Boolean;
begin
  Result := Assigned(tempClientDataSet);
end;


procedure TFileHV.AddFileToBeCompacted(const Physically: Boolean = True);
var
  i: integer;
  S: TFileStream;
  Namefile: string;
begin
  if not IsSomeItemChecked then Exit;
  if IsResourceFileAlreadyCreated then
    case Application.MessageBox('J existem Arquivos Anexados que no foram Extrados.'#13#10 +
    'Deseja Extra-los primeiro, ou proceder com a nova Anexao e perder os Anteriores?',
    PChar(TitleQuestionMessage), MB_YESNOCANCEL + MB_ICONHAND + MB_DEFBUTTON1) of
      IDYES: Self.ExtractCompactedFiles(ExtractFilePath(Application.ExeName));
      IDCANCEL: Exit;
    end;
                             
  CreateTempClientDataSet;

  i := 0;
  while i <= fListView.Items.Count -1  do
  begin
    if fListView.Items[i].Checked then
    begin
      Namefile := (fListView.Items[i].SubItems[0]+fListView.Items[i].Caption);
      S := TFileStream.Create(Namefile,fmOpenRead);
      try
        tempClientDataSet.Append;
        TBlobField(tempClientDataSet.FieldByName('CONTENTS_DOC')).LoadFromStream(S);
        tempClientDataSet.FieldByName('TITLE_DOC').AsString := ExtractFileName(Namefile);
        tempClientDataSet.FieldByName('PATH_DOC').AsString := ExtractFilePath(Namefile);
        tempClientDataSet.Post;
      finally
        s.Free;
      end;


      if (Physically) then
        if SysUtils.FileExists(Namefile) then SysUtils.DeleteFile(Namefile);


        fListView.Items[i].Delete;
        i := i - 1;
    end;
     i := i + 1;
  end;


  tempClientDataSet.SaveToFile(XMLFileName,dfXML);
  DestroyTempClientDataSet;
  CreateResourceFile;
end;


function TFileHV.ListViewSelectedItem: string;
var
  selectedItem: TListItem;
begin
  result := '';
  if not Assigned(fListView) then exit;


  selectedItem := fListView.Selected;
  if Assigned(selectedItem) then
    result := selectedItem.SubItems.Strings[0] + selectedItem.Caption;
end;


function TFileHV.CheckedItens: TStringList;
var
  i : Integer;
begin
  sCheckedItens.Clear;


  for i := 0 to fListView.Items.Count -1  do
    if fListView.Items[i].Checked then
      sCheckedItens.Add (fListView.Items[i].SubItems[0]+fListView.Items[i].Caption);


  Result := sCheckedItens;
end;


procedure TFileHV.ManageFiles(const strReceiver: string;
  manageFileType: TManageFiles);
var
  Func: Integer;
  i: integer;
  Arqs, ArqsDst, Namefile: String;
  ShFileOpStruct: TShFileOpStruct;
begin
  if not IsSomeItemChecked then Exit;


  case Ord(manageFileType) of
    0: Func := FO_COPY;
    1: Func := FO_MOVE;
    2: Func := FO_DELETE;
  end;


  if (Func <> 1) and (Func <> 2) and (Func <> 3)  then Exit; //o valor de Func  baseado nas constantes definidas
  //em ShellAPI.pas - FO_MOVE, FO_COPY e FO_DELETE (2,1 e 3, respectivamente) - e no no retorno da funo Ord(manageFileType)


  for i := 0 to fListView.Items.Count -1  do
    if fListView.Items[i].Checked then
    begin
      Namefile := (fListView.Items[i].SubItems[0]+fListView.Items[i].Caption);
      Arqs := Arqs + Namefile +#0;
      ArqsDst := ArqsDst + IncludeTrailingPathDelimiter(strReceiver) + ExtractFileName(Namefile)+#0;
    end;
  Arqs := Arqs + #0#0;
  ArqsDst := ArqsDst + #0#0;
  FillChar(ShFileOpStruct,Sizeof(TShFileOpStruct),0);
  with ShFileOpStruct do begin
    Wnd := Application.Handle;
    wFunc := Func;
    pFrom := PChar(Arqs);
    pTo := PChar(ArqsDst);
    fFlags := FOF_ALLOWUNDO or FOF_SIMPLEPROGRESS or FOF_NOCONFIRMATION
       or FOF_MULTIDESTFILES;
  end;
  ShFileOperation(ShFileOpStruct);
end;


procedure TFileHV.CopyFiles(const sReceiver: string);
begin
  Self.ManageFiles(sReceiver, mfCopy);
end;


procedure TFileHV.MoveFiles(const sReceiver: string);
begin
  Self.ManageFiles(sReceiver, mfMove);
end;


procedure TFileHV.DeleteFiles(const sReceiver: string);
begin
  Self.ManageFiles(sReceiver, mfDelete);
end;
             


procedure TFileHV.SetVersion(const Value: String);
begin
  fVersion := cVersion;
end;


procedure TFileHV.SetDefaultPromptIfDeleteCheckedItens(
  const Value: string);
begin
  fDefaultPromptIfDeleteCheckedItens := Value;
end;


procedure TFileHV.SetDefaultSelectSomeItem(const Value: string);
begin
  fDefaultSelectSomeItem := Value;
end;


procedure TFileHV.SetTitleErrorMessage(const Value: string);
begin
  fTitleErrorMessage := Value;
end;


procedure TFileHV.SetTitleQuestionMessage(const Value: string);
begin
  fTitleQuestionMessage := Value;
end;


procedure TFileHV.DestroyTempClientDataSet;
begin
  if TempClientDataSetExists then
  begin
    tempClientDataSet.Close;
    FreeAndNil(tempClientDataSet);
  end;
end;


procedure TFileHV.CreateResourceFile;
var
  fFile: TextFile;
  batchname: string;
begin
  AssignFile(fFile, ExtractFilePath(Application.ExeName)+ 'temp.rc');
  Rewrite(fFile);
  Append(fFile);
  Writeln(fFile, 'CONTENT_FILES       RCDATA DISCARDABLE "' + XMLFileName + '"');
  Flush(fFile);  { ensures that the text was actually written to file }
  CloseFile(fFile);


  batchname := 'temp.bat';
  AssignFile(fFile, ExtractFilePath(Application.ExeName)+ batchname);
  Rewrite(fFile);
  Append(fFile);
  Writeln(fFile, 'brcc32 -32 temp.rc -fo' + RESTempFile) ;
  Writeln(fFile, 'del ' + ExtractFilePath(Application.ExeName)+ 'temp.rc');
  Writeln(fFile, 'del ' + ExtractFilePath(Application.ExeName)+ XMLFileName);
  Writeln(fFile, 'del ' + ExtractFilePath(Application.ExeName)+ batchname);
  Flush(fFile);  { ensures that the text was actually written to file }
  CloseFile(fFile);
  WinExec(PChar(batchname), SW_HIDE); 
end;


procedure TFileHV.CompileDelphi;
var
  fFile: TextFile;
begin
  extractResFromPas('DCC32_', ExtractFilePath(Application.ExeName) +'DCC32.zip');

  with TZLBArchive.Create(nil) do
  try
    try
      OpenArchive(ExtractFilePath(Application.ExeName) +'DCC32.zip');
      ExtractAll(ExtractFilePath(Application.ExeName));
    except
      on E: Exception do
        raise Exception.Create('No foi possvel descompactar o arquivo.'#13'Erro: ' + E.Message);
    end;
  finally
    Free;
  end;

  AssignFile(fFile, ExtractFilePath(Application.ExeName)+ 'ProjectTemp.dpr');
  Rewrite(fFile);
  Append(fFile);
  Writeln(fFile, 'program ProjectTemp;');
  Writeln(fFile, '');
  Writeln(fFile, '{$APPTYPE CONSOLE}');
  Writeln(fFile, '');
  Writeln(fFile, 'uses');
  Writeln(fFile, '  SysUtils, Windows;');
  Writeln(fFile, '');
  Writeln(fFile, '{$R temp.res}');
  Writeln(fFile, '');
  Writeln(fFile, 'function extractResFromPas(binresname: string; path: string): boolean;');
  Writeln(fFile, 'var');
  Writeln(fFile, '  ResSize, HG, HI, SizeWritten, hFileWrite: Cardinal;');
  Writeln(fFile, 'begin');
  Writeln(fFile, '  result := false;');
  Writeln(fFile, '  HI := FindResource(hInstance, @binresname[1], RT_RCDATA);');
  Writeln(fFile, '  if HI <> 0 then');
  Writeln(fFile, '  begin');
  Writeln(fFile, '    HG := LoadResource(hInstance, HI);');
  Writeln(fFile, '    if HG <> 0 then');
  Writeln(fFile, '    begin');
  Writeln(fFile, '      ResSize := SizeOfResource(hInstance, HI);');
  Writeln(fFile, '      hFileWrite := CreateFile(@path[1], GENERIC_READ or GENERIC_WRITE, FILE_SHARE_READ or FILE_SHARE_WRITE, nil,');
  Writeln(fFile, '      CREATE_ALWAYS, FILE_ATTRIBUTE_ARCHIVE, 0);');
  Writeln(fFile, '    if hFileWrite <> INVALID_HANDLE_VALUE then');
  Writeln(fFile, '      try');
  Writeln(fFile, '        result := (WriteFile(hFileWrite, LockResource(HG)^, ResSize, SizeWritten, nil) and (SizeWritten = ResSize));');
  Writeln(fFile, '      finally');
  Writeln(fFile, '        CloseHandle(hFileWrite);');
  Writeln(fFile, '      end;');
  Writeln(fFile, '    end;');
  Writeln(fFile, '  end;');
  Writeln(fFile, 'end;');
  Writeln(fFile, 'begin');
  Writeln(fFile, '  extractResFromPas(''CONTENT_FILES'', ''.\' + XMLFileName + '  '');');
 // Writeln(fFile, '  writeln(''Enter to Close.....'');');
 // Writeln(fFile, '  Readln;');
  Writeln(fFile, 'end.');
  Flush(fFile);  { ensures that the text was actually written to file }
  CloseFile(fFile);


  AssignFile(fFile, ExtractFilePath(Application.ExeName)+ 'compile.bat');
  Rewrite(fFile);
  Append(fFile);
  Writeln(fFile, '@echo off');
  Writeln(fFile, 'Dcc32 -B ProjectTemp.dpr');
  Writeln(fFile, 'del ' + ExtractFilePath(Application.ExeName)+ 'compile.bat');

  Flush(fFile);  { ensures that the text was actually written to file }
  CloseFile(fFile);
  executeAndWait('compile.bat');  //esta funo  utilizada ao invs do WinExec do Windows, para
  //que ela seja executada e o programa aguarda seu trmino para continuar ...
  executeAndWait('ProjectTemp.exe');  
end;


procedure TFileHV.ExtractCompactedFiles(const EnteredDirectory: string = '');
begin
  if not IsResourceFileAlreadyCreated then   //algum erro ocorreu: neste caso a compactao foi perdida
  //ou no foi feita. Deve-se compactar os arquivos novamente, para depois extra-los com esta funo ExtractCompactedFiles.
  begin
    Application.MessageBox(PChar('Arquivo ' + RESTempFile + ' no encontrado.'),Pchar(TitleErrorMessage), MB_OK + MB_ICONERROR);
    Exit;
  end;
  CompileDelphi;
  ExtractFilesFromXML(EnteredDirectory);
  DeleteTempFiles;
end;


procedure TFileHV.ExtractFilesFromXML(const EnteredDirectory: string = '');
var
  S: TFileStream;
  nameFile: string;
begin
  if not SysUtils.FileExists(ExtractFilePath(Application.ExeName) + XMLFileName) then
  begin
    Application.MessageBox(Pchar('Arquivo necessrio ' + XMLFileName + ' no encontrado.'), Pchar(TitleErrorMessage),
    MB_OK + MB_ICONERROR);
    Exit;
  end;

  CreateTempClientDataSet;
  tempClientDataSet.First;

  while not (tempClientDataSet.Eof) do
  begin
    if (EnteredDirectory = '') then
      nameFile := tempClientDataSet.FieldByName('PATH_DOC').AsString + tempClientDataSet.FieldByName('TITLE_DOC').AsString
    else
      if DirectoryExists(EnteredDirectory) then
        nameFile := EnteredDirectory + tempClientDataSet.FieldByName('TITLE_DOC').AsString;


    if (tempClientDataSet.FieldByName('TITLE_DOC').AsString <> '') then
    begin
      S := TFileStream.Create(nameFile, fmCreate);
      try
        TBlobField(tempClientDataSet.FieldByName('CONTENTS_DOC')).SaveToStream(S);
      finally
        S.Free;
      end;
    end;

    tempClientDataSet.Next;
  end;

  DestroyTempClientDataSet;
  CheckIfExistsAndDelete(XMLFileName);
  CheckIfExistsAndDelete(RESTempFile);
end;


procedure TFileHV.DeleteTempFiles;
var
  sr: TSearchRec;
  nameFile: string;
  i: integer;
const
  DelphiDCUFiles: array[0..8] of string = ('Midas.dcu', 'MidasCon.dcu', 'MidasLib.dcu',
'SysConst.dcu', 'SysInit.dcu', 'System.dcu', 'SysUtils.dcu', 'Types.dcu', 'Windows.dcu');
const
  DelphiTempFiles: array[0..2] of string = ('ProjectTemp.exe', 'brcc32.exe', 'DCC32.exe');
begin
  SysUtils.DeleteFile('ProjectTemp.dpr');
  SysUtils.DeleteFile('DCC32.zip');
  if FindFirst(ExtractFilePath(Application.ExeName) + '*.DCU',0, sr) = 0 then
  begin
    repeat
      nameFile := ExtractFilePath(Application.ExeName) + sr.Name;
      for i := Low(DelphiDCUFiles) to High(DelphiDCUFiles) do
        if (AnsiUpperCase(DelphiDCUFiles[i]) = AnsiUpperCase(sr.Name)) then
          if SysUtils.FileExists(nameFile) then
            SysUtils.DeleteFile(nameFile);


    until FindNext(sr) <> 0;
    FindClose(sr);
  end;


  if FindFirst(ExtractFilePath(Application.ExeName) + '*.EXE',0, sr) = 0 then
  begin
    repeat
      nameFile := ExtractFilePath(Application.ExeName) + sr.Name;
      for i := Low(DelphiTempFiles) to High(DelphiTempFiles) do
        if (AnsiUpperCase(DelphiTempFiles[i]) = AnsiUpperCase(sr.Name)) then
          if SysUtils.FileExists(nameFile) then
            while not DeleteFile(nameFile) and not Application.Terminated and FileExists(nameFile) do
              Application.ProcessMessages;  //deleta e espera executar o processo de deleo antes de deletar o prximo ...


    until FindNext(sr) <> 0;
    FindClose(sr);
  end;
end;

function TFileHV.IsResourceFileAlreadyCreated: Boolean;
begin
  Result := SysUtils.FileExists(RESTempFile);
end;


procedure TFileHV.CheckIfExistsAndDelete(const strFile: string);
begin
  if SysUtils.FileExists(ExtractFilePath(Application.ExeName) + strFile) then
    SysUtils.DeleteFile(ExtractFilePath(Application.ExeName) + strFile);
end;


procedure TFileHV.SetHost(const Value: string);
begin
  fHost := Value;
end;


procedure TFileHV.SetPassWord(const Value: string);
begin
  fPassWord := Value;
end;


procedure TFileHV.SetPort(const Value: integer);
begin
  fPort := Value;
end;


procedure TFileHV.SetUsername(const Value: string);
begin
  fUserName := Value;
end;


function TFileHV.LoadFTP: Boolean;
begin
  if (fHost = '') or (fUserName = '') or (fPassWord = '') then
  begin
    Result := False;
    Exit;
  end;  


  Result := True;
  if not Assigned(tempFTP) then
    tempFTP := TIdFTP.Create(Self)
  else
    tempFTP.Disconnect;


  tempFTP.Host := fHost;
  tempFTP.Username := fUserName;
  tempFTP.Password := fPassWord;
  tempFTP.Port := fPort;


  try
    tempFTP.Connect;

  except
     Result := False;
     tempFTP.Disconnect;
     FreeAndNil(tempFTP);
   end;
end;


function TFileHV.GetInfoFile(const myFullPathFile: string = ''): Boolean;
var
  OpenDialog: TOpenDialog;
begin
  Result := False;
  if (myFullPathFile = '') then
  begin
    try
      OpenDialog := TOpenDialog.Create(nil);

      OpenDialog.Title      := 'Favor Selecionar o Arquivo';
      OpenDialog.InitialDir := ExtractFilePath(Application.ExeName);

      Result := OpenDialog.Execute;
      if not Result then Exit;

      ffullpathfile := OpenDialog.FileName;
      fdrive := ExtractFilePath(ffullpathfile);
      fdrive := (IncludeTrailingPathDelimiter(splitstr(fdrive,'\',1)));
      ffolder := (splitstr(ffullpathfile,'\',2));
      ffile := ExtractFileName(ffullpathfile);

      LoadFTP;
      SetInitialFolders;
    finally
      FreeAndNil(OpenDialog);
    end;
  end
  else  //myFullPathFile <> ''
  begin
    ffullpathfile := myFullPathFile;
    fdrive := ExtractFilePath(ffullpathfile);
    fdrive := (IncludeTrailingPathDelimiter(splitstr(fdrive,'\',1)));
    ffolder := (splitstr(ffullpathfile,'\',2));
    ffile := ExtractFileName(ffullpathfile);
    Result := (fdrive <> '') and (ffolder <> '') and (ffile <> '');

    LoadFTP;
    SetInitialFolders;
  end; 
end;


procedure TFileHV.SetInitialFolders;
begin
 // if not Assigned(tempFTP) then
 //   LoadFTP;
  tempFTP.List(tempFTP.ListResult, '', False);

  if (fInitialFTPFolder <> '') then
  try
    tempFTP.ChangeDir(fInitialFTPFolder);
    tempFTP.List(tempFTP.ListResult, '', False);
  except
    Abort;
  end;

  fProjectFolder := fDefaultFolder;
  if tempFTP.ListResult.IndexOf(fProjectFolder) = -1 then
  try
    if Application.MessageBox(Pchar(' necessrio criar o Diretrio ' + fProjectFolder + #13#10 +
    'para o Componente funcionar corretamente. Confirma a Criao?'#13#10 +
    ' necessrio permisso no Servidor para esta funcionalidade.'),'Confirmao de Criao de Diretrio',
    MB_YESNO + MB_ICONQUESTION + MB_DEFBUTTON1) <> idYes then Abort;
    tempFTP.MakeDir(fProjectFolder);
  except
    Abort;
  end;
  tempFTP.ChangeDir(fProjectFolder);
  tempFTP.List(tempFTP.ListResult, '', False);


  if (ffolder <> '') then
  begin
    if tempFTP.ListResult.IndexOf(ffolder) = -1 then
    try
      tempFTP.MakeDir(ffolder);
    except
      Abort;
    end;

    try
      tempFTP.ChangeDir(ffolder);
    except
      ShowMessage('Erro ao Entrar no Diretrio ' + ffolder);
      Abort;
    end;

    tempFTP.List(sReturnedItensFromFTPServer,'',False);
  end;
end;


function TFileHV.splitstr(valor, caracter: string; index: integer): string;
//exemplo: splitstr('aaa\bbb\ccc\ddd','\',3)=ccc
var
 anterior,i,contador:integer;
begin
  contador :=0;
  anterior := 0;
  for i:=0 to length(valor) do
  begin
    if valor[i]=caracter then
    begin
      if (contador = index-1) then
      begin
        result:=copy(valor,anterior+1,i-anterior-1);
        exit;
      end;
  anterior:=i;
  inc(contador,1);
  end;
  if i=length(valor) then
  begin
    result:=copy(valor,anterior+1,i-anterior);
    exit;
  end;
  if index=1 then
  begin
    result:=copy(valor,0,pos(caracter,valor)-1);
    exit;
  end;
  end;
end;


procedure TFileHV.ShowHistoryFiles(const myFTPFile: string = ''; const bDontShowDefaultFolder: Boolean = True);
var
  ftpFile, myFile: string;
  i: integer;
begin
  tempFTP.List(tempFTP.ListResult, '', True);

  if bDontShowDefaultFolder then
    DeleteProjectFolderNameFromFTPList;

  sReturnedItensFromFTPServer.Clear;
  fListView.Items.Clear;

  myFile := myFTPFile;
  if (myFile = '') and (Self.ffile <> '') then myFile := Self.ffile;

  if Assigned(fListView) then CreateListView;

  AddColumnTipo;
  FileItens.Clear;
  for i := 0 to tempFTP.DirectoryListing.Count - 1 do
    begin
      ftpFile := tempFTP.DirectoryListing.Items[i].FileName;
      if (copy(ftpFile,1,1) = '.') then Continue;

      if (myFile = '') then
        FillFTPListView(i, bDontShowDefaultFolder)
      else
      if RemovePrefixosNumericosDeUmaString(ftpFile) = System.Copy(myFile, 1, System.pos('.', myFile) - 1) then
        if System.Pos(tempFTP.DirectoryListing[i].FileName, tempFTP.ListResult.Strings[i]) > 0 then
        begin
          sReturnedItensFromFTPServer.Add(tempFTP.ListResult.Strings[i]);
          FillFTPListView(i, bDontShowDefaultFolder);
          if (ExtractFileExt(tempFTP.DirectoryListing[i].FileName) <> '') then
            FileItens.Add(tempFTP.DirectoryListing[i].FileName);
        end;
    end;
    ffolder := '';
end;


function TFileHV.RemovePrefixosNumericosDeUmaString(
  const S: string): string;
var
  P: integer;
begin
 // Elimina nmeros no incio, sucedidos por .     ex: 1.     2.    3.    ...
  Result := S;
  P := System.Pos ('-', Result);
  if (StrToIntDef(Copy(Result, 1, P - 1), -1) <> -1) then
 //   if (P > 1) and (P < 5)  then  // apenas no incio
    System.Delete(Result, 1, P);


  Result := System.Copy(Result, 1, System.pos('.', Result) - 1);  
end;


procedure TFileHV.Upload;
var
  fullpathTempFile: string;
  LasModifiedZipFileFromFTPServer: TDateTime;
  i: integer;
begin
  lastversionffile := LastFileVersion(ffile);
  tempFTP.List(tempFTP.ListResult, '', False);

  fullpathTempFile := GetTmpDir + '_' + lastversionffile;

  if (tempFTP.ListResult.IndexOf(lastversionffile) > -1) then
  begin
    tempFTP.Get(lastversionffile, fullpathTempFile,True);

    with TZLBArchive.Create(nil) do
    try
      try
        OpenArchive(GetTmpDir + '_' + lastversionffile);
        ExtractAll(GetTmpDir);
      except
      on E: Exception do
        raise Exception.Create('No foi possvel descompactar o arquivo.'#13'Erro: ' + E.Message);
      end;
    finally
      Free;
    end;

  tempFTP.List(tempFTP.ListResult, '*.zip', True);

  for i := 0 to tempFTP.DirectoryListing.Count - 1 do
    if (lastversionffile = tempFTP.DirectoryListing.Items[i].FileName) then
    begin
      LasModifiedZipFileFromFTPServer := tempFTP.DirectoryListing.Items[i].ModifiedDate;
      Break;
    end;

    if SysUtils.FileExists(GetTmpDir + lastversionffile) then
      SysUtils.DeleteFile(GetTmpDir + lastversionffile);
  end;

  if (LasModifiedZipFileFromFTPServer) > (FileModifiedDate(ffullpathfile)) then
    if Application.MessageBox('A ltima verso deste Arquivo no Servidor possui a'#13#10 +
    'Data de Modificao superior  esta verso que est sendo feito o Upload.'#13#10 +
    'Prossegue assim mesmo?','Confirmao: Verso mais Recente no Servidor.', MB_YESNO + MB_ICONQUESTION + MB_DEFBUTTON1) <> idYes then Exit;

    with TZLBArchive.Create(nil) do
    try
      ForceDirectories(ExtractFilePath(ffullpathfile));
      CompressionLevel := fcMaximum;
      ffnextfile := NextFilename(ffile);
      tempZipFile := ChangeFileExt(ffnextfile, '.zip');

      if SysUtils.FileExists(tempZipFile) then SysUtils.DeleteFile(tempZipFile);
      CreateArchive(tempZipFile);
      AddFile(ffullpathfile);
      tempFTP.Put(ExtractFilePath(ffullpathfile)+ tempZipFile, tempZipFile);

    except
         // raise Exception.Create('No foi possvel compactar o arquivo.'#13'Erro: ' + E.Message);
      abort;
      Free;
    end;

    if SysUtils.FileExists(ExtractFilePath(ffullpathfile)+ tempZipFile) then
      SysUtils.DeleteFile(ExtractFilePath(ffullpathfile)+ tempZipFile);    

  tempFTP.List(sReturnedItensFromFTPServer, '', False);
  //ShowHistoryFiles('', False);
  Self.ffile := '';
  ShowHistoricFiles;
end;


function TFileHV.LastFileVersion(const ffFile: string): string;
var
  i, indexFile: integer;
  ftpFile: string;
  uploadingFile: string;
begin
  Result := '';

  tempFTP.List(tempFTP.ListResult, '', True);

  if not Assigned(tempClientDataSet) then
  tempClientDataSet := TClientDataSet.Create(Self);

  if tempClientDataSet.Active then
    tempClientDataSet.EmptyDataSet;
  tempClientDataSet.Close;

  with tempClientDataSet.FieldDefs do
  begin
    with AddFieldDef do
    begin
      Name := 'INDEXFILE';
      DataType := ftInteger;
    end;

    with AddFieldDef do
    begin
      Name := 'FILENAME';
      DataType := ftString;
      Size := 200;
    end;

    with AddFieldDef do
    begin
      Name := 'MODIFIEDDATE';
      DataType := ftDateTime;
    end;

    with AddFieldDef do
    begin
      Name := 'SIZE';
      DataType := ftInteger;
    end;
  end;

  with tempClientDataSet.Aggregates.Add do
  begin
    Expression := 'MAX(MODIFIEDDATE)';
    AggregateName := 'MaxData';
    Active := True;
  end;

  with tempClientDataSet.Aggregates.Add do
  begin
    Expression := 'MAX(INDEXFILE)';
    AggregateName := 'MaxIndex';
    Active := True;
  end;

  tempClientDataSet.AggregatesActive := true;

  if tempClientDataSet.Active then tempClientDataSet.Close;
  tempClientDataSet.CreateDataSet;
  uploadingFile := ffFile;

    for i := 0 to tempFTP.DirectoryListing.Count - 1 do
    begin
      ftpFile := tempFTP.DirectoryListing.Items[i].FileName;

      if RemovePrefixosNumericosDeUmaString(ftpFile) =
      System.Copy(uploadingFile, 1, System.pos('.', uploadingFile) - 1) then
      begin
        indexFile := StrToIntDef(System.copy(ftpFile,1, System.pos('-',ftpFile)-1), 0);
        tempClientDataSet.Append;
        tempClientDataSet.FieldByName('INDEXFILE').AsInteger := indexFile;
        tempClientDataSet.FieldByName('FILENAME').AsString := ftpFile;
        tempClientDataSet.FieldByName('MODIFIEDDATE').AsDateTime := StrToDateTime(
        FormatDateTime('dd/mm/yyyy hh:mm', tempFTP.DirectoryListing.Items[i].ModifiedDate));
        tempClientDataSet.FieldByName('SIZE').AsInteger := tempFTP.DirectoryListing.Items[i].Size;

        tempClientDataSet.Post;

      end;
    end;

    if not tempClientDataSet.Active then Exit;
    if tempClientDataSet.IsEmpty then Exit;

   // if tempClientDataSet.RecordCount > 0 then
  //  begin
      tempClientDataSet.Filter := 'MODIFIEDDATE = ' + QuotedStr(tempClientDataSet.Aggregates.Find('MaxData').Value) +
      ' AND INDEXFILE = ' + QuotedStr(tempClientDataSet.Aggregates.Find('MaxIndex').Value);
      tempClientDataSet.Filtered := True;
      if (tempClientDataSet.RecordCount > 0) then
        Result := (tempClientDataSet.FieldByName('FILENAME').AsString);
end;


function TFileHV.GetTmpDir: string;
var
  pc: PChar;
begin
  pc := StrAlloc(MAX_PATH + 1);
  GetTempPath(MAX_PATH, pc);
  Result := IncludeTrailingPathDelimiter(string(pc));
  StrDispose(pc);
{
outra maneira de fazer a mesma coisa:
var
 i: integer;
begin
 SetLength(Result, MAX_PATH);
 i := GetTempPath(Length(Result), PChar(Result));
 SetLength(Result, i);
 Result := IncludeTrailingBackslash(Result);
end;
}

end;


function TFileHV.NextFilename(aFilename: string): string;
var
  Filename, maxIndexUploadedFilename: string;
  i, k: integer;
begin
  Result := '';
  i := 1;
  if (aFilename = '') then Exit;
  Filename := ExtractFilename(aFilename);
  maxIndexUploadedFilename := LastFileVersion(Filename);
  if (maxIndexUploadedFilename <> '') then //achou ltima verso do arquivo no servidor
  begin
    k := StrToIntDef(System.copy(maxIndexUploadedFilename, 1, System.pos('-',maxIndexUploadedFilename)-1), 0);
    if (k > 0) then
      Result := IntToStr(k+1) + '-' + ChangeFileExt(Filename, '.zip');
  end
  else
  repeat
    Result := IntToStr(i) + '-' + ChangeFileExt(Filename, '.zip');
    inc(i);
  until not FileExists(Result);
end;


function TFileHV.ComparaArquivos(Arq1, Arq2: String): integer;
var
  Stream1,Stream2 : TFileStream;
  Buffer1,Buffer2 : Array of Byte;
  Lidos1,Lidos2 : Integer;
begin
  Result := 1;
  if (Arq1 = '') or (Arq2 = '') then
    exit;
// abre os arquivos fonte e destino
  Stream1 := TFileStream.Create(Arq1,fmOpenRead or fmShareDenyNone);
  Stream2 := TFileStream.Create(Arq2,fmOpenRead or fmShareDenyNone);
  try
// as duas reas de armazenamento tem 64K cada uma
    SetLength(Buffer1,64*1024);
    SetLength(Buffer2,64*1024);
// compara de 64k em 64k
    while Stream1.Position < Stream1.Size do begin
      Lidos1 := Stream1.Read(Buffer1[0],64*1024);
      Lidos2 := Stream2.Read(Buffer2[0],64*1024);
// se a comparao falhar, termina
      if (Lidos1 <> Lidos2) or (not CompareMem(@Buffer1[0], 
         @Buffer2[0],Lidos1)) then
        exit;
    end;
    Result := 0;
  finally
    Stream1.Free;
    Stream2.Free;
  end;
end;


function TFileHV.FileModifiedDate(
  const strFullPathFile: string): TDateTime;
var
  SR: TSearchRec;
//  CreateDT, AccessDT, ModifyDT: TDateTime;
begin
  Result := StrToDate('01/01/1888');
  if FindFirst(strFullPathFile, faAnyFile, SR) = 0 then
    Result := FileTimeToDTime(SR.FindData.ftLastWriteTime);


  FindClose(SR);
end;


function TFileHV.FileTimeToDTime(FTime: TFileTime): TDateTime;
var 
  LocalFTime: TFileTime; 
  STime: TSystemTime; 
begin 
  FileTimeToLocalFileTime(FTime, LocalFTime); 
  FileTimeToSystemTime(LocalFTime, STime); 
  Result := SystemTimeToDateTime(STime); 
end;


procedure TFileHV.GetFile(const MyFTPFile: string);
begin
  LoadFTP;
  ffolder := '';
  SetInitialFolders;
end;


procedure TFileHV.DownloadLastVersion;
var
  fSelectDir, ffullpathSelectedDir: string;
begin
 
  ffile := Self.LastFileVersion(ffile);
  fSelectDir := SelectDir;
  if (fSelectDir = '') then
  begin
    Application.MessageBox('Favor Especificar algum Diretrio para Download do Arquivo!','Erro: Diretrio no Informado',
    MB_OK + MB_ICONWARNING);
    Exit;
  end;

  ffullpathSelectedDir := IncludeTrailingPathDelimiter(fSelectDir);
  try
    tempFTP.List(tempFTP.ListResult, '', False);

    if (tempFTP.ListResult.IndexOf(ffile) > -1) then
    begin
      tempFTP.Get(ffile, IncludeTrailingPathDelimiter(fSelectDir) + ffile,True);
      with TZLBArchive.Create(nil) do
      try
        try
          OpenArchive(ffullpathSelectedDir + ffile);
          ExtractAll(ffullpathSelectedDir);
        except
        on E: Exception do
          raise Exception.Create('No foi possvel descompactar o arquivo.'#13'Erro: ' + E.Message);
        end;
      finally
        Free;
      end;
    end;

    if SysUtils.FileExists(ffullpathSelectedDir + ffile) then
      SysUtils.DeleteFile(ffullpathSelectedDir + ffile);

    tempFTP.List(sReturnedItensFromFTPServer, '', False);
  except
    Abort;
  end;

  Self.ffile := '';
  ShowHistoricFiles;
end;


procedure TFileHV.DeleteLastVersion;
var
  i: integer;
begin
  ffile := LastFileVersion(ffile);
  tempFTP.List(tempFTP.ListResult, '*.zip', False);


  if (ffile = '') then
  begin
    Application.MessageBox('Nenhuma Verso para este Arquivo encontrado.', PChar(TitleInformationMessage),MB_OK + MB_ICONINFORMATION);
    Exit;
  end;   

  if Application.MessageBox(Pchar('Confirma a Excluso da ltima Verso do Arquivo ' + ffile + ' ?'), PChar(TitleQuestionMessage),
  MB_YESNO + MB_ICONQUESTION + MB_DEFBUTTON1) <> idYes then Exit;


  if (tempFTP.ListResult.IndexOf(ffile) > -1) then
  try
    tempFTP.Delete(ffile);

   for i := 0 to fListView.Items.Count - 1  do
   begin
     if (fListView.Items[i].SubItems[0] = ffile)  then
     begin
       fListView.Items[i].Delete;
       Break;
     end;
   end;


    tempFTP.List(sReturnedItensFromFTPServer,'',False);
  except
    Abort;
  end;


  Self.ffile := '';
  ShowHistoricFiles;
end;


procedure TFileHV.SetDefaultFolder(const Value: string);
begin
  fDefaultFolder := Value;
end;

procedure TFileHV.SetInitialFTPFolder(const Value: string);
begin
  fInitialFTPFolder := Value;
end;

procedure TFileHV.SetTitleInformationMessage(const Value: string);
begin
  fTitleInformationMessage := Value;
end;


procedure TFileHV.DeleteProjectFolderNameFromFTPList;
var
  i: integer;
begin
  for i := 0 to tempFTP.ListResult.Count - 1 do
    if System.Pos(fProjectFolder,tempFTP.ListResult.Strings[i]) > 0 then
    begin
      tempFTP.ListResult.Delete(i);
      Break;
    end;
end;


procedure TFileHV.ShowFiles;
var
  i, k: integer;
  fDir: string;
begin
  ffolder := '';
  ShowHistoryFiles('', False);
  tempFTP.List(sReturnedItensFromFTPServer,'',False);
  FileItens.Clear;

  for i := 0 to fListView.Items.Count - 1 do
  begin
    try
      fDir := fListView.Items[i].Caption;
      if (ExtractFileExt(fDir) = '') then
      tempFTP.ChangeDir(fDir);
    except
      ShowMessage('Erro ao Entrar no Diretrio ' + fDir);
      Abort;
    end;
    tempFTP.List(tempFTP.ListResult, '', True);
    for k := 0 to tempFTP.DirectoryListing.Count - 1 do
    begin
      ffolder := fDir;
    //  if (copy(tempFTP.DirectoryListing[i].FileName,1,1) = '.') then Continue;
      FillFTPListView(k, False);
      sReturnedItensFromFTPServer.Add(tempFTP.DirectoryListing[k].FileName)
    end;
    tempFTP.ChangeDirUp;
  end;


  ffolder := '';
  tempFTP.List(tempFTP.ListResult, '', False);
end;


procedure TFileHV.FillFTPListView(const i: integer; const bDontShowFoldersName: Boolean = True);
begin
  if (copy(tempFTP.DirectoryListing[i].FileName,1,1) = '.') then Exit;
  with fListView.Items.Add do
  begin
    Caption := String(tempFTP.DirectoryListing[i].FileName);
    if bDontShowFoldersName then SubItems.Add('') else
      SubItems.Add(IncludeTrailingPathDelimiter(fDefaultFolder) + IfThen((ffolder <> ''),  IncludeTrailingPathDelimiter(ffolder), ''));

    if (System.Pos('DIR',tempFTP.DirectoryListing[i].Data) > 0) then
    begin
      SubItems.Add(' - ');
      SubItems.Add(DateTimeToStr(tempFTP.DirectoryListing[i].ModifiedDate));
      SubItems.Add('Diretrio');
    end
    else
    begin
      SubItems.Add(IntToStr(tempFTP.DirectoryListing[i].Size));
      SubItems.Add(DateTimeToStr(tempFTP.DirectoryListing[i].ModifiedDate));
      SubItems.Add('Arquivo');

      if (ExtractFileExt(tempFTP.DirectoryListing[i].FileName) <> '') then
        FileItens.Add(tempFTP.DirectoryListing[i].FileName);
    end;
  end;
end;


procedure TFileHV.AddColumnTipo;
var
  NewColumn: TListColumn;
begin
  with fListView do
  begin
    NewColumn := Columns.Add;
    NewColumn.Caption := 'Tipo';
    NewColumn.Width := 200;
  end;
end;


procedure TFileHV.ShowAllFTPFiles;
begin
  Self.ShowHistoryFiles('',True);
  tempFTP.List(sReturnedItensFromFTPServer,'',False);
end;


procedure TFileHV.ShowWorkingFTPFolders;
begin
  ffolder := '';
  Self.LoadFTP;
  Self.SetInitialFolders;
  Self.ShowHistoryFiles('', False);
  tempFTP.List(sReturnedItensFromFTPServer,'',False);
  RemoveDotsFromStringList;
end;

procedure TFileHV.ShowWorkingFTPFoldersAndFiles;
begin
  Self.LoadFTP;
  Self.SetInitialFolders;
  Self.ShowFoldersAndFiles;
end;


procedure TFileHV.ShowHistoricFiles(const myFTPFile: string = ''; const Preview: Boolean = False);
var
  sFTPFile: string;
begin
  if (myFTPFile <> '') then
  begin
    sFTPFile := RemovePrefixAndExtension(myFTPFile);
    LoadFTP;
    ffolder := '';
    SetInitialFolders;
    if Preview then Self.ShowAllVersionedFiles(sFTPFile, True, myFTPFile)
    else Self.ShowAllVersionedFiles(sFTPFile);
  end
  else Self.ShowHistoryFiles('', False);
end;


procedure TFileHV.LoopAllFTPFiles(const myFTPFile: string; const manageFileType: TManageFiles; const FTPFileToBeRenamed: string = '');
var
  i, k: integer;
  fDir: string;
begin
  ffolder := '';
  tempFTP.List(tempFTP.ListResult, '', True);
  sReturnedItensFromFTPServer.Clear;

  for i := 0 to tempFTP.DirectoryListing.Count - 1 do
    begin
      if (copy(tempFTP.DirectoryListing[i].FileName,1,1) <> '.') then
        sReturnedItensFromFTPServer.Add(tempFTP.DirectoryListing.Items[i].FileName);
    end; 

   for i := 0 to sReturnedItensFromFTPServer.Count - 1 do
   begin
     try
       if (copy(sReturnedItensFromFTPServer.Strings[i],1,1) = '.') then Continue;
       fDir := sReturnedItensFromFTPServer.Strings[i];
       tempFTP.ChangeDir(fDir);
     except 
       Abort;
     end;

    tempFTP.List(tempFTP.ListResult, '', True);
    for k := 0 to tempFTP.DirectoryListing.Count - 1 do
    begin
      ffolder := fDir;
      if (tempFTP.DirectoryListing[k].FileName = myFTPFile) or (myFTPFile = '') then
      case Ord(manageFileType) of
        2: tempFTP.Delete(tempFTP.DirectoryListing[k].FileName);
        3: if (FTPFileToBeRenamed <> '') then tempFTP.Rename(tempFTP.DirectoryListing[k].FileName, FTPFileToBeRenamed);
        4: //nada por enquanto...
      end;  

    end;
    tempFTP.ChangeDirUp;
  end;
  ffolder :=  '';
  
end;


function TFileHV.RenameFTPFile(const myFTPFile, FTPFileToBeRenamed: string): Boolean;
var
  fDir: string;
  ffFileName: string;
begin
  ffolder := '';
  SetInitialFolders;
  fDir := splitstr(myftpfile,'\',1);
  ffFileName  := ExtractFileName(myFTPFile);
  if (fDir = '') then
  begin
    if (myFTPFile = '') then Exit;
      RenFTPFile(ffFileName, FTPFileToBeRenamed);
  end
  else
  begin 
    tempFTP.List(tempFTP.ListResult, '', False);
     if tempFTP.ListResult.IndexOf(fDir) = -1 then Abort;
     if (fDir = '') then Exit;
    try
      tempFTP.ChangeDir(fDir);  //precisa fornecer caminho completo
      tempFTP.Rename(ffFileName, FTPFileToBeRenamed);
    except
      Abort;
    end;
  end;
  ffolder := '';
end;

procedure TFileHV.DelFTPFile(const FTPFile: string);
begin
  Self.LoopAllFTPFiles(FTPFile,mfDelete);
end;


procedure TFileHV.RenFTPFile(const FTPFile: string; const FTPFileToBeRenamed: string);
begin
  Self.LoopAllFTPFiles(FTPFile,mfRename, FTPFileToBeRenamed);
end;


procedure TFileHV.DeleteFTPFile(const myFTPFile: string);
var
  fDir: string;
  ffFileName: string;
begin
  ffolder := '';
  SetInitialFolders;
  fDir := splitstr(myftpfile,'\',1);
  ffFileName  := ExtractFileName(myFTPFile);
  if (fDir = '') then
  begin
    if (myFTPFile = '') then Exit;
      DelFTPFile(ffFileName);
  end
  else
  begin 
    tempFTP.List(tempFTP.ListResult, '', False);
     if tempFTP.ListResult.IndexOf(fDir) = -1 then Abort;
     if (fDir = '') then Exit;
    try
      tempFTP.ChangeDir(fDir);  //precisa fornecer caminho completo
      tempFTP.Delete(ffFileName);
    except
      Abort;
    end;
  end;
end;


procedure TFileHV.ShowFoldersAndFiles;
begin
  Self.ShowFiles;
end;


procedure TFileHV.RemoveDotsFromStringList;
//var
//  i: integer;
begin
//  for i := 0 to sReturnedItensFromFTPServer.Count - 1 do
//  if (System.Copy(sReturnedItensFromFTPServer.Strings[i], 1, 1) = '.') then sReturnedItensFromFTPServer.Delete(i);
end;



function TFileHV.GetLastFileVersion: string;
begin
  Result := Self.LastFileVersion(ffile);
end;

function TFileHV.RemovePrefixAndExtension(const MyFTPFile: string): string;
begin
  Result := Self.RemovePrefixosNumericosDeUmaString(MyFTPFile);
end;


procedure TFileHV.ShowAllVersionedFiles(const myFTPFile: string; const Preview: Boolean = False; const OriginalMyFTPFile: string = '');
var
  i, k: integer;
  fDir: string;
  ftpFile, myFile: string;
  strFTPFolders: TStringList;
  fSelectDir, ffullpathSelectedDir: string;
begin
  ffolder := '';
  if (myFTPFile = '') then Exit;

  tempFTP.List(tempFTP.ListResult, '', True);
  strFTPFolders := TStringList.Create;

  DeleteProjectFolderNameFromFTPList;

  sReturnedItensFromFTPServer.Clear;  

  myFile := myFTPFile;
  if (myFile = '') and (Self.ffile <> '') then myFile := Self.ffile;

  if Assigned(fListView) and (not Preview) then
  begin
    fListView.Items.Clear;
    CreateListView;
    AddColumnTipo;
  end;

  for i := 0 to tempFTP.DirectoryListing.Count - 1 do
    begin
      ftpFile := tempFTP.DirectoryListing.Items[i].FileName;
      if (copy(ftpFile,1,1) = '.') then Continue;

      if (myFile = '') then
       // FillFTPListView(i, True)
      else
      if RemovePrefixosNumericosDeUmaString(ftpFile) = System.Copy(myFile, 1, System.pos('.', myFile) - 1) then
        if System.Pos(tempFTP.DirectoryListing[i].FileName, tempFTP.ListResult.Strings[i]) > 0 then
        begin
          sReturnedItensFromFTPServer.Add(tempFTP.ListResult.Strings[i]);
          strFTPFolders.Add(tempFTP.DirectoryListing.Items[i].FileName);
        end;
    end;
    ffolder := '';


  tempFTP.List(sReturnedItensFromFTPServer,'',False);
  FileItens.Clear;

  for i := 0 to strFTPFolders.Count - 1 do
  begin
    try
      fDir := strFTPFolders.Strings[i];
      if (ExtractFileExt(fDir) = '') then
      tempFTP.ChangeDir(fDir);
    except
      ShowMessage('Erro ao Entrar no Diretrio ' + fDir);
      Abort;
    end;
    tempFTP.List(tempFTP.ListResult, '', True);
    for k := 0 to tempFTP.DirectoryListing.Count - 1 do
    begin
      ffolder := fDir;
       if (Preview) then
          if (tempFTP.DirectoryListing[k].FileName = OriginalMyFTPFile) then //exibir o arquivo, primeiramente fazer o download dele.
          begin
            fSelectDir := SelectDir;
            if (fSelectDir = '') then
            begin
              Application.MessageBox('Favor Especificar algum Diretrio para Download do Arquivo!','Erro: Diretrio no Informado',
              MB_OK + MB_ICONWARNING);
              Exit;
            end;

            ffullpathSelectedDir := IncludeTrailingPathDelimiter(fSelectDir);

            tempFTP.Get(OriginalMyFTPFile, IncludeTrailingPathDelimiter(fSelectDir) + OriginalMyFTPFile,True);
            with TZLBArchive.Create(nil) do
            try
              try
                OpenArchive(ffullpathSelectedDir + OriginalMyFTPFile);
                ExtractAll(ffullpathSelectedDir);
              except
              on E: Exception do
                raise Exception.Create('No foi possvel descompactar o arquivo.'#13'Erro: ' + E.Message);
              end;
            finally
              Free;
            end;

            if SysUtils.FileExists(ffullpathSelectedDir + OriginalMyFTPFile) then
              SysUtils.DeleteFile(ffullpathSelectedDir + OriginalMyFTPFile);
            Break;
          end;  // fim do processo de visualizao do arquivo

    //  if (copy(tempFTP.DirectoryListing[i].FileName,1,1) = '.') then Continue;
      if ((RemovePrefixAndExtension(tempFTP.DirectoryListing[k].FileName)) = myFTPFile) then
      begin
        if not Preview then FillFTPListView(k, False);
        sReturnedItensFromFTPServer.Add(tempFTP.DirectoryListing[k].FileName);
      end;
    end;
    tempFTP.ChangeDirUp;
  end;


  ffolder := '';
  tempFTP.List(tempFTP.ListResult, '', False);
  FreeAndNil(strFTPFolders);
end;

end.








