unit UCadArquivo;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, Grids, DBGrids, StdCtrls, DBCtrls,
  Buttons;

type
  TfrmCadArquivo = class(TForm)
    p1: TPanel;
    bbAddArq: TBitBtn;
    bbDelArq: TBitBtn;
    lstArquivos: TListBox;
    pBotoes: TPanel;
    bbImportar: TBitBtn;
    p2: TPanel;
    bbAbrir: TBitBtn;
    bbDelTabela: TBitBtn;
    bbSalvar: TBitBtn;
    bbSalvaT: TBitBtn;
    p4: TPanel;
    gbx1: TGroupBox;
    p3: TPanel;
    bbObs: TBitBtn;
    memObs: TDBMemo;
    Splitter1: TSplitter;
    gbx2: TGroupBox;
    img: TImage;
    bbFechar: TBitBtn;
    SaveDialog: TSaveDialog;
    DBGrid1: TDBGrid;
    procedure bbAddArqClick(Sender: TObject);
    procedure bbDelArqClick(Sender: TObject);
    procedure bbFecharClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure bbImportarClick(Sender: TObject);
    procedure memObsKeyPress(Sender: TObject; var Key: Char);
    procedure bbObsClick(Sender: TObject);
  private
    { Private declarations }
    FPessoa : Integer;  // armazena o cdigo da pessoa que ser passada por parmetros

    Procedure Gravar;
  public
    { Public declarations }
    // Class Procedure para chamar a janela
    Class Procedure Executar(APessoa : Integer);
  end;

Const
  // "\\<NOME DO COMPUTADOR>\<PASTA COMPARTILHADA$>\"
  COMPARTILHAMENTO = '\\MICRO1\ANEXOS$\';

var
  frmCadArquivo: TfrmCadArquivo;

implementation

uses uDM;

{$R *.dfm}

procedure TfrmCadArquivo.bbAddArqClick(Sender: TObject);
// evento para adicionar os arquivos no ListBox
Var
  sArquivo : String;
begin
  // abre a janela para escolher os arquivos
  If SaveDialog.Execute Then
  Begin
    // guardo o path + nome do arquivo
    sArquivo := SaveDialog.FileName;
    // adiciona no ListBox
    lstArquivos.Items.Add(sArquivo);
    // habilita o boto "Retirar da Seleo"
    bbDelArq.Enabled   := True;
    // habilita o boto "Importar"
    bbImportar.Enabled := True;
  End;
end;


procedure TfrmCadArquivo.bbDelArqClick(Sender: TObject);
// deleta a linha do ListBox
begin
  // se o list box estiver vazio
  If lstArquivos.ItemIndex = -1 Then
    Exit; // sai e no faz nada

  // apaga a linha do ListBox
  lstArquivos.Items.Delete(lstArquivos.ItemIndex);
  // se o ListBox fica vazio aps a excluso...
  If lstArquivos.Items.Count = 0 Then
  Begin
    // desabilito os botes "Importar" e
    // "Retirar da seleo"
    bbImportar.Enabled  := False;
    bbDelArq.Enabled    := False;
  End;
end;

procedure TfrmCadArquivo.bbFecharClick(Sender: TObject);
begin
  dm.cdsArquivo.Close;
  Close;
end;

procedure TfrmCadArquivo.bbImportarClick(Sender: TObject);
begin
  if (MessageDlg('Confirma o(s) arquivos(s) selecionado(s)?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
  Begin
    Gravar;
  End;
end;

procedure TfrmCadArquivo.bbObsClick(Sender: TObject);
begin
  bbObs.Enabled := False;
  dm.cdsArquivo.ApplyUpdates(0);
end;

class procedure TfrmCadArquivo.Executar(APessoa: Integer);
// class procedure para chamada desta janela
begin
  Try
    frmCadArquivo := TfrmCadArquivo.Create(nil);
    // cdigo da pessoa passada por parametros
    frmCadArquivo.FPessoa := APessoa;
    // mostra a janeja
    frmCadArquivo.ShowModal;
  Finally
    FreeAndNil(frmCadArquivo);
  End;
end;

procedure TfrmCadArquivo.FormShow(Sender: TObject);
begin
  // abre a tabela com os registros da pessoa
  dm.cdsArquivo.Close;
  dm.cdsArquivo.Params.ParamByName('ID_PESSOA').AsInteger := FPessoa;
  dm.cdsArquivo.Open;
end;

procedure TfrmCadArquivo.Gravar;
Var
  i, iTotErro : Integer;
  sExt, sTXTErro, sErro, sOrigem, sDestino : String;

  Function CopiaArquivo(AOrigem, ADestino : String; var AErro : String) : Boolean;
  // copia o arquivo
  begin
    Result := True;
    // verifica se o arq existe
    If not FileExists(AOrigem) Then
    Begin
      Result := False;
      AErro := 'O arquivo no existe no sistema.';
    End
    // copia o arquivo
    Else If not CopyFile(PChar(AOrigem),PChar(ADestino),false) Then
    Begin
      Result := False;
      AErro := 'Falha ao copiar o arquivo.';
    End;
  end;


begin
  iTotErro := 0;
  sTXTErro := '';
  Repeat
    Try
      i := 0;
      // nome original do arquivo
      sOrigem  := lstArquivos.Items.Strings[i];

      dm.cdsArquivo.Append;
      dm.cdsArquivoID_PESSOA.AsInteger    := FPessoa;
      dm.cdsArquivoDATA_CRIACAO.AsDateTime:= date;
      dm.cdsArquivoNOME_ORIGINAL.AsString := ExtractFileName(sOrigem);
      // como o campo NOME_ARQUIVO  obrigatrio,
      // gravo a string TMP para depois gravar o nome corretamente
      dm.cdsArquivoNOME_ARQUIVO.AsString  := 'TMP';
      If dm.cdsArquivo.ApplyUpdates(0) > 0 Then
      Begin
        Raise Exception.Create('Falha ao criar o registro');
      End;
      // atualizo o ClientDataSet para pegar o ID do arquivo
      dm.cdsArquivo.Close;
      dm.cdsArquivo.Open;
      dm.cdsArquivo.Last; // ltimo registro

      // somente o nome original do arquivo+extenso
      sDestino := ExtractFileName(sOrigem);
      // somente a extenso do arquivo original
      sExt     := ExtractFileExt(sOrigem);
      // o nome do arquivo de destino ser o ID da tabela Arquivo +
      // a extenso do nome do arquivo original
      sDestino := trim(IntToStr(dm.cdsArquivoID_ARQUIVO.AsInteger))+sExt;
      // o arq de destino ter o nome da pasta compartilhada +
      // o arquivo renomeado
      sDestino := COMPARTILHAMENTO+sDestino;

      // copio o arquivo para a pasta compartilhada
      If not CopiaArquivo(sOrigem,sDestino,sErro) Then
      Begin
        // se ocorram erros ao copiar o arquivo
        // apago o registro criado
        dm.cdsArquivo.Delete;
        dm.cdsArquivo.ApplyUpdates(0);
        // retorno uma exceo
        Raise Exception.Create(sErro);
      End
      Else
      Begin
        dm.cdsArquivo.Edit;
        // agora sim, gravo o nome do arquivo destino corretamente
        dm.cdsArquivoNOME_ARQUIVO.AsString := ExtractFileName(sDestino);
        dm.cdsArquivo.ApplyUpdates(0);
      End;
      // apago a linha do ListBox
      lstArquivos.Items.Delete(0);
    Except
      On E : Exception Do
      Begin
        // apago a linha do ListBox
        lstArquivos.Items.Delete(0);
        // qtd de erros ocorridos
        Inc(iTotErro);
        // cancela a gravao
        dm.cdsArquivo.CancelUpdates;
        sTXTErro := sTXTErro + sOrigem + ' - ' + e.Message +#13+#10;
      End;
    End;
  Until (lstArquivos.Count = 0);
  If iTotErro > 0 Then
  Begin
    // se ocorreram erros...
    MessageDlg(Format('%d arquivo(s) no foram importados.',[iTotErro])+#13+#10+
               sTXTErro, mtWarning, [mbOK], 0);
  End
  Else
  Begin
    // tudo certo
    MessageDlg('Arquivo(s) importados com sucesso!', mtInformation, [mbOK], 0);
  End;
end;

procedure TfrmCadArquivo.memObsKeyPress(Sender: TObject; var Key: Char);
begin
  bbObs.Enabled := True;
end;

end.
